@@ -7,13 +7,16 @@ from typing import Any, Dict, Optional, Tuple
 from uuid import UUID
 
 import numpy
-
 from datacube.utils.geometry import Geometry
 from toolz import get_in
 from urlpath import URL
 
 from ._docs import odc_uuid
 
+from eodatasets3.serialise import from_doc
+from eodatasets3.stac import to_stac_item
+from datacube.model import Dataset
+
 Document = Dict[str, Any]
 
 # This is an old hack, should be refactored out
@@ -21,7 +24,12 @@ DEA_LANDSAT_PRODUCTS = ["ga_ls8c_ard_3", "ga_ls7e_ard_3", "ga_ls8t_ard_3"]
 
 # This is hack for not changing the current behavior of DEAfrica sentinel-2
 # It is not ideal but to remain the impact minimum
-TO_BE_HARD_CODED_COLLECTION = ["s2_l2a", "sentinel_s2_l2a_cogs", "sentinel-s2-l2a-cogs"]
+TO_BE_HARD_CODED_COLLECTION = [
+    "s2_l2a",
+    "sentinel_s2_l2a_cogs",
+    "sentinel-s2-l2a-cogs",
+    "sentinel-2-l2a",
+]
 
 # Mapping between EO3 field names and STAC properties object field names
 MAPPING_STAC_TO_EO3 = {
@@ -112,12 +120,21 @@ def _stac_product_lookup(
             )
             product_name = "s2_l2a"
             if region_code is None:
-                # Let this throw an exception if there's something missing
-                region_code = (
-                    f"{str(properties['proj:epsg'])[-2:]}"
-                    f"{properties['sentinel:latitude_band']}"
-                    f"{properties['sentinel:grid_square']}"
-                )
+                # Let's try two options, and throw an exception if we still don't get it
+                try:
+                    # The 'mgrs' prefix (and STAC extension) started with STAC v1.0.0
+                    region_code = (
+                        f"{str(properties['proj:epsg'])[-2:]}"
+                        f"{properties['mgrs:latitude_band']}"
+                        f"{properties['mgrs:grid_square']}"
+                    )
+                except KeyError:
+                    region_code = (
+                        f"{str(properties['proj:epsg'])[-2:]}"
+                        f"{properties['sentinel:latitude_band']}"
+                        f"{properties['sentinel:grid_square']}"
+                    )
+
             default_grid = "g10m"
 
     # If we still don't have a product name, use collection
@@ -404,3 +421,20 @@ def stac_transform(input_stac: Document, relative: bool = True) -> Document:
 def transform_geom_json_coordinates_to_list(geom_json):
     geom_json["coordinates"] = numpy.array(geom_json["coordinates"]).tolist()
     return geom_json
+
+
+def ds_to_stac(ds: Dataset) -> dict:
+    """Get STAC document from dataset with eo3 metadata"""
+    if ds.is_eo3:
+        if not ds.uris:
+            raise ValueError("Can't find dataset location")
+        location = ds.uris[0]
+        stac = to_stac_item(
+            from_doc(ds.metadata_doc, skip_validation=True),
+            location,
+        )
+        return stac
+    else:
+        raise ValueError(
+            f"Cannot convert to STAC for dataset with metadata of type {ds.metadata_type}"
+        )
