@@ -9,27 +9,40 @@ from pathlib import PurePath
 from typing import Tuple
 
 import boto3
-from botocore import UNSIGNED
-from botocore.config import Config
 import click
 import pandas as pd
 import requests
+from botocore import UNSIGNED
+from botocore.config import Config
 from datacube import Datacube
 from datacube.index.hl import Doc2Dataset
 from datacube.utils import documents
-from odc.apps.dc_tools.utils import (IndexingException, SkippedException, allow_unsafe, archive,
-                                     fail_on_missing_lineage,
-                                     index_update_dataset, limit, no_sign_request,
-                                     skip_lineage,
-                                     statsd_setting, statsd_gauge_reporting,
-                                     transform_stac, transform_stac_absolute,
-                                     archive_less_mature,
-                                     update, update_if_exists, verify_lineage)
-from odc.aws.queue import get_messages
-from ._stac import stac_transform, stac_transform_absolute
+from odc.apps.dc_tools.utils import (
+    IndexingException,
+    SkippedException,
+    allow_unsafe,
+    archive,
+    fail_on_missing_lineage,
+    index_update_dataset,
+    limit,
+    no_sign_request,
+    skip_lineage,
+    statsd_setting,
+    statsd_gauge_reporting,
+    transform_stac,
+    transform_stac_absolute,
+    archive_less_mature,
+    update_flag,
+    update_if_exists_flag,
+    verify_lineage,
+    publish_action,
+)
+from odc.aws.queue import get_messages, publish_to_topic
 from toolz import dicttoolz
 from yaml import safe_load
 
+from ._stac import stac_transform, stac_transform_absolute, ds_to_stac
+
 # Added log handler
 logging.basicConfig(level=logging.WARNING, handlers=[logging.StreamHandler()])
 
@@ -117,10 +130,11 @@ def handle_bucket_notification_message(
             # If you specific a list of record paths, and there's no
             # match in them for the key, then we skip this one forever
             if record_path is not None and not any(
-                [PurePath(key).match(p) for p in record_path]
+                PurePath(key).match(p) for p in record_path
             ):
                 logging.warning(
-                    f"Key: {key} not in specified list of record_paths, deleting message from the queue."
+                    "Key: %s not in specified list of record_paths, deleting message from the queue.",
+                    key,
                 )
                 # This will return Nones, which will flag the message to be ignored
                 return None, None
@@ -130,18 +144,20 @@ def handle_bucket_notification_message(
             try:
                 uri = f"s3://{bucket_name}/{key}"
                 if no_sign_request:
-                    s3 = boto3.client('s3', config=Config(signature_version=UNSIGNED))
+                    s3 = boto3.client("s3", config=Config(signature_version=UNSIGNED))
                     data = s3.get_object(Bucket=bucket_name, Key=key)
-                    contents = data['Body'].read()
+                    contents = data["Body"].read()
                     data = safe_load(contents.decode("utf-8"))
                 else:
                     s3 = boto3.resource("s3")
-                    obj = s3.Object(bucket_name, key).get(ResponseCacheControl="no-cache")
+                    obj = s3.Object(bucket_name, key).get(
+                        ResponseCacheControl="no-cache"
+                    )
                     data = safe_load(obj["Body"].read())
             except Exception as e:
                 raise IndexingException(
-                    f"Exception thrown when trying to load s3 object: {e}"
-                )
+                    "Exception thrown when trying to load s3 object"
+                ) from e
     else:
         raise IndexingException(
             "Attempted to get metadata from record when no record key exists in message."
@@ -159,10 +175,16 @@ def get_uri(metadata, rel_value):
     return uri
 
 
-def do_archiving(metadata, dc: Datacube):
+def do_archiving(metadata, dc: Datacube, publish_action):
     dataset_id = uuid.UUID(metadata.get("id"))
     if dataset_id:
         dc.index.datasets.archive([dataset_id])
+        if publish_action:
+            publish_to_topic(
+                arn=publish_action,
+                action="ARCHIVED",
+                stac=ds_to_stac(dc.index.datasets.get(dataset_id)),
+            )
     else:
         raise IndexingException("Failed to get an ID from the message, can't archive.")
 
@@ -182,8 +204,9 @@ def queue_to_odc(
     odc_metadata_link=False,
     region_code_list_uri=None,
     archive_less_mature=None,
+    publish_action=None,
     **kwargs,
-) -> Tuple[int, int]:
+) -> Tuple[int, int, int]:
 
     ds_success = 0
     ds_failed = 0
@@ -195,8 +218,8 @@ def queue_to_odc(
             region_codes = set(
                 pd.read_csv(region_code_list_uri, header=None).values.ravel()
             )
-        except FileNotFoundError as e:
-            logging.error(f"Could not find region_code file with error: {e}")
+        except FileNotFoundError:
+            logging.exception("Could not find region_code file")
         if len(region_codes) == 0:
             raise IndexingException(
                 f"Region code list is empty, please check the list at: {region_code_list_uri}"
@@ -211,11 +234,14 @@ def queue_to_odc(
         try:
             # Extract metadata from message
             metadata = extract_metadata_from_message(message)
+            stac_doc = None
             if archive:
                 # Archive metadata
-                do_archiving(metadata, dc)
+                do_archiving(metadata, dc, publish_action)
             else:
                 if not record_path:
+                    if transform:
+                        stac_doc = metadata
                     # Extract metadata and URI from a STAC or similar
                     # json structure for indexing
                     metadata, uri = handle_json_message(
@@ -239,7 +265,8 @@ def queue_to_odc(
                         metadata = None
                         uri = None
                         logging.warning(
-                            f"Region code {region_code} not in list of allowed region codes, ignoring this dataset."
+                            "Region code %s not in list of allowed region codes, ignoring this dataset.",
+                            region_code,
                         )
 
                 # Index the dataset
@@ -253,10 +280,12 @@ def queue_to_odc(
                             update=update,
                             update_if_exists=update_if_exists,
                             allow_unsafe=allow_unsafe,
-                            archive_less_mature=archive_less_mature
+                            archive_less_mature=archive_less_mature,
+                            publish_action=publish_action,
+                            stac_doc=stac_doc,
                         )
                         ds_success += 1
-                    except (SkippedException) as e:
+                    except SkippedException:
                         ds_skipped += 1
                 else:
                     logging.warning("Found None for metadata and uri, skipping")
@@ -264,8 +293,8 @@ def queue_to_odc(
 
             # Success, so delete the message.
             message.delete()
-        except (IndexingException) as err:
-            logging.exception(f"Failed to handle message with exception: {err}")
+        except IndexingException:
+            logging.exception("Failed to handle SQS message")
             ds_failed += 1
 
     return ds_success, ds_failed, ds_skipped
@@ -277,8 +306,8 @@ def queue_to_odc(
 @verify_lineage
 @transform_stac
 @transform_stac_absolute
-@update
-@update_if_exists
+@update_flag
+@update_if_exists_flag
 @allow_unsafe
 @archive
 @limit
@@ -305,6 +334,7 @@ def queue_to_odc(
     help="A path to a list (one item per line, in txt or gzip format) of valide region_codes to include",
 )
 @archive_less_mature
+@publish_action
 @click.argument("queue_name", type=str, nargs=1)
 @click.argument("product", type=str, nargs=1)
 def cli(
@@ -324,10 +354,11 @@ def cli(
     record_path,
     region_code_list_uri,
     archive_less_mature,
+    publish_action,
     queue_name,
     product,
 ):
-    """ Iterate through messages on an SQS queue and add them to datacube"""
+    """Iterate through messages on an SQS queue and add them to datacube"""
 
     transform = None
     if stac:
@@ -361,27 +392,38 @@ def cli(
         odc_metadata_link=odc_metadata_link,
         region_code_list_uri=region_code_list_uri,
         archive_less_mature=archive_less_mature,
+        publish_action=publish_action,
     )
 
     result_msg = ""
     if update:
         result_msg += f"Updated {success} Dataset(s), "
         if statsd_setting:
-            statsd_gauge_reporting(success, ["app:sqs_to_dc", "action:update"], statsd_setting)
+            statsd_gauge_reporting(
+                success, ["app:sqs_to_dc", "action:update"], statsd_setting
+            )
     elif archive:
         result_msg += f"Archived {success} Dataset(s), "
         if statsd_setting:
-            statsd_gauge_reporting(success, ["app:sqs_to_dc", "action:archived"], statsd_setting)
+            statsd_gauge_reporting(
+                success, ["app:sqs_to_dc", "action:archived"], statsd_setting
+            )
     else:
         result_msg += f"Added {success} Dataset(s), "
         if statsd_setting:
-            statsd_gauge_reporting(success, ["app:sqs_to_dc", "action:added"], statsd_setting)
+            statsd_gauge_reporting(
+                success, ["app:sqs_to_dc", "action:added"], statsd_setting
+            )
     result_msg += f"Failed {failed} Dataset(s), Skipped {skipped} Dataset(s)"
     print(result_msg)
 
     if statsd_setting:
-        statsd_gauge_reporting(failed, ["app:sqs_to_dc", "action:failed"], statsd_setting)
-        statsd_gauge_reporting(skipped, ["app:sqs_to_dc", "action:skipped"], statsd_setting)
+        statsd_gauge_reporting(
+            failed, ["app:sqs_to_dc", "action:failed"], statsd_setting
+        )
+        statsd_gauge_reporting(
+            skipped, ["app:sqs_to_dc", "action:skipped"], statsd_setting
+        )
 
     if failed > 0:
         sys.exit(failed)
