@@ -13,11 +13,18 @@ import pystac
 from datacube import Datacube
 from datacube.index.hl import Doc2Dataset
 from odc.apps.dc_tools._stac import stac_transform, stac_transform_absolute
-from odc.apps.dc_tools.utils import (SkippedException, allow_unsafe,
-                                     archive_less_mature, bbox,
-                                     index_update_dataset, limit,
-                                     statsd_gauge_reporting, statsd_setting,
-                                     update_if_exists)
+from odc.apps.dc_tools.utils import (
+    SkippedException,
+    allow_unsafe,
+    archive_less_mature,
+    bbox,
+    index_update_dataset,
+    limit,
+    statsd_gauge_reporting,
+    statsd_setting,
+    update_if_exists_flag,
+    publish_action,
+)
 from pystac.item import Item
 from pystac_client import Client
 
@@ -38,12 +45,18 @@ def _parse_options(options: Optional[str]) -> Dict[str, Any]:
 
                 try:
                     value = json.loads(value)
-                except Exception:
-                    logging.warning(f"Failed to handle value {value} for key {key} as JSON, using str")
+                except Exception:  # pylint:disable=broad-except
+                    logging.warning(
+                        "Failed to handle value %s for key %s as JSON, using str",
+                        value,
+                        key,
+                    )
                 parsed_options[key] = value
-            except Exception as e:
+            except Exception:  # pylint:disable=broad-except
                 logging.warning(
-                    f"Couldn't parse option {option}, format is key=value, exception was {e}"
+                    "Couldn't parse option %s format is key=value",
+                    option,
+                    exc_info=True,
                 )
 
     return parsed_options
@@ -94,13 +107,14 @@ def item_to_meta_uri(
     metadata = item.to_dict()
     if rename_product is not None:
         metadata["properties"]["odc:product"] = rename_product
+    stac = metadata
 
     if relative:
         metadata = stac_transform(metadata)
     else:
         metadata = stac_transform_absolute(metadata)
 
-    return (metadata, uri)
+    return (metadata, uri, stac)
 
 
 def process_item(
@@ -112,8 +126,9 @@ def process_item(
     rewrite: Optional[Tuple[str, str]] = None,
     rename_product: Optional[str] = None,
     archive_less_mature: bool = False,
+    publish_action: bool = False,
 ):
-    meta, uri = item_to_meta_uri(item, rewrite, rename_product)
+    meta, uri, stac = item_to_meta_uri(item, rewrite, rename_product)
     index_update_dataset(
         meta,
         uri,
@@ -122,6 +137,8 @@ def process_item(
         update_if_exists=update_if_exists,
         allow_unsafe=allow_unsafe,
         archive_less_mature=archive_less_mature,
+        publish_action=publish_action,
+        stac_doc=stac,
     )
 
 
@@ -134,6 +151,7 @@ def stac_api_to_odc(
     rewrite: Optional[Tuple[str, str]] = None,
     rename_product: Optional[str] = None,
     archive_less_mature: bool = False,
+    publish_action: Optional[str] = None,
 ) -> Tuple[int, int, int]:
     doc2ds = Doc2Dataset(dc.index)
     client = Client.open(catalog_href)
@@ -141,10 +159,10 @@ def stac_api_to_odc(
     search = client.search(**config)
     n_items = search.matched()
     if n_items is not None:
-        logging.info("Found {} items to index".format(n_items))
+        logging.info("Found %s items to index", n_items)
         if n_items == 0:
             logging.warning("Didn't find any items, finishing.")
-            return 0, 0
+            return 0, 0, 0
     else:
         logging.warning("API did not return the number of items.")
 
@@ -166,6 +184,7 @@ def stac_api_to_odc(
                 rewrite=rewrite,
                 rename_product=rename_product,
                 archive_less_mature=archive_less_mature,
+                publish_action=publish_action,
             ): item.id
             for item in search.get_all_items()
         }
@@ -177,10 +196,9 @@ def stac_api_to_odc(
                 if success % 10 == 0:
                     sys.stdout.write(f"\rAdded {success} datasets...")
             except SkippedException:
-                logging.exception(f"{item} Skipped")
                 skipped += 1
-            except Exception as e:
-                logging.exception(f"Failed to handle item {item} with exception {e}")
+            except Exception:  # pylint:disable=broad-except
+                logging.exception("Failed to handle item %s", item)
                 failure += 1
     sys.stdout.write("\r")
 
@@ -189,7 +207,7 @@ def stac_api_to_odc(
 
 @click.command("stac-to-dc")
 @limit
-@update_if_exists
+@update_if_exists_flag
 @allow_unsafe
 @click.option(
     "--catalog-href",
@@ -235,6 +253,7 @@ def stac_api_to_odc(
     ),
 )
 @archive_less_mature
+@publish_action
 @statsd_setting
 def cli(
     limit,
@@ -249,6 +268,7 @@ def cli(
     rename_product,
     statsd_setting,
     archive_less_mature,
+    publish_action,
 ):
     """
     Iterate through STAC items from a STAC API and add them to datacube.
@@ -288,6 +308,7 @@ def cli(
         rewrite=rewrite,
         rename_product=rename_product,
         archive_less_mature=archive_less_mature,
+        publish_action=publish_action,
     )
 
     print(
