@@ -1,16 +1,17 @@
-import os
 import logging
-import click
-import pkg_resources
-
+import os
 from typing import Iterable, Optional, Union
 
+import click
+import pkg_resources
 from datacube import Datacube
 from datacube.index.hl import Doc2Dataset
 from datacube.utils import changes
 
 from datadog import statsd, initialize
 
+from odc.aws.queue import publish_to_topic
+from ._stac import ds_to_stac
 
 ESRI_LANDCOVER_BASE_URI = (
     "https://ai4edataeuwest.blob.core.windows.net/io-lulc/"
@@ -25,15 +26,12 @@ class IndexingException(Exception):
     Exception to raise for error during SQS to DC indexing/archiving
     """
 
-    pass
-
 
 class SkippedException(Exception):
     """
     Exception to raise for error if dataset exists  and not updating
     """
 
-    pass
 
 # A whole bunch of generic Click options
 skip_lineage = click.option(
@@ -74,14 +72,14 @@ transform_stac_absolute = click.option(
     help="Use absolute paths from the STAC document.",
 )
 
-update = click.option(
+update_flag = click.option(
     "--update",
     is_flag=True,
     default=False,
     help="If set, update instead of add datasets.",
 )
 
-update_if_exists = click.option(
+update_if_exists_flag = click.option(
     "--update-if-exists",
     is_flag=True,
     default=False,
@@ -103,7 +101,10 @@ skip_check = click.option(
 )
 
 no_sign_request = click.option(
-    "--no-sign-request", is_flag=True, default=False, help="Do not sign AWS S3 requests."
+    "--no-sign-request",
+    is_flag=True,
+    default=False,
+    help="Do not sign AWS S3 requests.",
 )
 
 request_payer = click.option(
@@ -117,8 +118,20 @@ archive_less_mature = click.option(
     "--archive-less-mature",
     is_flag=True,
     default=False,
-    help="""Archive existing any datasets that match product, time and region-code, but have lower dataset-maturity.
-Note: An error will be raised and the dataset add will fail if a matching dataset with higher or equal dataset-maturity."""
+    help=(
+        "Archive existing any datasets that match product, "
+        "time and region-code, but have lower dataset-maturity."
+        "Note: An error will be raised and the dataset add will "
+        "fail if a matching dataset with higher or equal dataset-maturity."
+    ),
+)
+
+publish_action = click.option(
+    "--publish-action",
+    type=str,
+    default=None,
+    nargs=1,
+    help="SNS topic arn to publish indexing/archiving actions to.",
 )
 
 archive = click.option(
@@ -146,7 +159,7 @@ statsd_setting = click.option(
     "--statsd-setting",
     is_flag=False,
     default=None,
-    help="statsd exporter hostname and port, i.e. prometheus-statsd-exporter:9125"
+    help="statsd exporter hostname and port, i.e. prometheus-statsd-exporter:9125",
 )
 
 
@@ -154,8 +167,8 @@ def get_esri_list():
     stream = pkg_resources.resource_stream(__name__, "esri-lc-tiles-list.txt")
     with stream as f:
         for tile in f.readlines():
-            id = tile.decode().rstrip('\n')
-            yield ESRI_LANDCOVER_BASE_URI.format(id=id)
+            tile_id = tile.decode().rstrip("\n")
+            yield ESRI_LANDCOVER_BASE_URI.format(id=tile_id)
 
 
 def index_update_dataset(
@@ -166,7 +179,9 @@ def index_update_dataset(
     update: bool = False,
     update_if_exists: bool = False,
     allow_unsafe: bool = False,
-    archive_less_mature: Optional[Union[bool, Iterable[str]]]=None
+    archive_less_mature: Optional[Union[bool, Iterable[str]]] = None,
+    publish_action: Optional[str] = None,
+    stac_doc: Optional[dict] = None,
 ) -> int:
     """
     Index and/or update a dataset.  Called by all the **_to_dc CLI tools.
@@ -186,6 +201,8 @@ def index_update_dataset(
            * If an iterable of valid search field names is provided, it is used as the "grouping" fields for
              identifying dataset maturity matches.
              (i.e. `archive_less_mature=True` is the same as `archive_less_mature=['region_code', 'time'])
+    :param publish_action: SNS topic arn to publish action to.
+    :param stac_doc: STAC document for publication to SNS topic.
     :return: Returns nothing.  Raises an exception if anything goes wrong.
     """
     if uri is None:
@@ -196,14 +213,14 @@ def index_update_dataset(
     except ValueError as e:
         raise IndexingException(
             f"Exception thrown when trying to create dataset: '{e}'\n The URI was {uri}"
-        )
+        ) from e
     if ds is None:
         raise IndexingException(
             f"Failed to create dataset with error {err}\n The URI was {uri}"
         )
 
     if archive_less_mature:
-        if archive_less_mature == True:
+        if archive_less_mature is True:
             # if set explicitly to True, default to [region_code, time]
             archive_less_mature = ["region_code", "time"]
         try:
@@ -218,13 +235,11 @@ def index_update_dataset(
     with dc.index.transaction():
         # Process in a transaction
         archive_ids = []
+        archive_stacs = []
         added = False
         updated = False
         if archive_less_mature:
-            dupes = dc.index.datasets.search(
-                product=ds.type.name,
-                **dupe_query
-            )
+            dupes = dc.index.datasets.search(product=ds.type.name, **dupe_query)
             for dupe in dupes:
                 if dupe.id == ds.id:
                     # Same dataset, for update.  Ignore
@@ -233,10 +248,13 @@ def index_update_dataset(
                     # Duplicate is as mature, or more mature than ds
                     # E.g. "final" < "nrt"
                     raise IndexingException(
-                        f"Matching dataset of maturity {dupe.metadata.dataset_maturity} already exists (id: {dupe.id})\n"
-                        f" Cannot load dataset of maturity {ds.metadata.dataset_maturity} URI {uri} "
+                        f"Matching dataset of maturity {dupe.metadata.dataset_maturity} already exists"
+                        "with id: {dupe.id}\n"
+                        f"Cannot load dataset of maturity {ds.metadata.dataset_maturity} URI {uri} "
                     )
                 archive_ids.append(dupe.id)
+                if publish_action:
+                    archive_stacs.append(ds_to_stac(dupe))
             if archive_ids:
                 dc.index.datasets.archive(archive_ids)
 
@@ -275,24 +293,29 @@ def index_update_dataset(
     # Transaction committed : Log actions
     for arch_id in archive_ids:
         logging.info("Archived less mature dataset: %s", arch_id)
+    if publish_action:
+        for arch_stac in archive_stacs:
+            publish_to_topic(arn=publish_action, action="ARCHIVED", stac=arch_stac)
+
     if added:
         logging.info("New Dataset Added: %s", ds.id)
+        if publish_action:
+            # if STAC was not provided, generate from dataset
+            stac_doc = stac_doc if stac_doc else ds_to_stac(ds)
+            publish_to_topic(arn=publish_action, action="ADDED", stac=stac_doc)
+
     if updated:
         logging.info("Existing Dataset Updated: %s", ds.id)
 
 
-def statsd_gauge_reporting(
-    value, tags=[],
-    statsd_setting="localhost:8125"
-):
+def statsd_gauge_reporting(value, tags=None, statsd_setting="localhost:8125"):
+    if tags is None:
+        tags = []
     host = statsd_setting.split(":")[0]
     port = statsd_setting.split(":")[1]
-    options = {
-        'statsd_host': host,
-        'statsd_port': port
-    }
+    options = {"statsd_host": host, "statsd_port": port}
     initialize(**options)
 
     if os.environ.get("HOSTNAME"):
         tags.append(f"pod:{os.getenv('HOSTNAME')}")
-    statsd.gauge('datacube_index', value, tags=tags)
\ No newline at end of file
+    statsd.gauge("datacube_index", value, tags=tags)
